package com.example.gasullajproyectomaster;

import android.graphics.drawable.Drawable;
import android.view.View;

public class Bloque extends Grafico {
	
	// Indica el tipo de bloque, se le asigna una de las 
	// variables indicadas a continuacin
	private int tipoBloque;
	
	// Tipos de bloque indicando en que lados del bloque puede colisionar la
	// pelota (N: ninguno, L: izquierda, D: derecha, U: arriba, D: abajo
    public static final int TIPO_BLOQUE_N    = 0;
    public static final int TIPO_BLOQUE_L    = 1;
    public static final int TIPO_BLOQUE_R    = 2;
    public static final int TIPO_BLOQUE_U    = 3;
    public static final int TIPO_BLOQUE_D    = 4;
    public static final int TIPO_BLOQUE_LR   = 5;
    public static final int TIPO_BLOQUE_LU   = 6;
    public static final int TIPO_BLOQUE_LD   = 7;
    public static final int TIPO_BLOQUE_LRU  = 8;
    public static final int TIPO_BLOQUE_LRD  = 9;
    public static final int TIPO_BLOQUE_LRUD = 10;
    public static final int TIPO_BLOQUE_LUD  = 11;
    public static final int TIPO_BLOQUE_RU   = 12;
    public static final int TIPO_BLOQUE_RD   = 13;
    public static final int TIPO_BLOQUE_RUD  = 14;
    public static final int TIPO_BLOQUE_UD   = 15;    

    public Bloque(View view, Drawable drawable, int tipoBloque) {
        super(view, drawable);
        this.tipoBloque = tipoBloque;
    }

    // Indica en cual de los cuatro lados del bloque
    // ha colisionado el Grafico g, o en ninguno
    public void verificaColisionLados(Grafico g) {
        // Inicialmente indicamos que no hay colisin
        g.setColisionX(Grafico.COLISION_NO);
        g.setColisionY(Grafico.COLISION_NO);

        // Lmites del bloque
        int leftBloque   = this.getCenX() - this.getAncho()/2;
        int topBloque    = this.getCenY() - this.getAlto()/2;
        int rightBloque  = this.getCenX() + this.getAncho()/2;
        int bottomBloque = this.getCenY() + this.getAlto()/2;

        // Lmites de la pelota (grfico g)
        int leftPelota   = g.getCenX() - g.getAncho()/2;
        int topPelota    = g.getCenY() - g.getAlto()/2;
        int rightPelota  = g.getCenX() + g.getAncho()/2;
        int bottomPelota = g.getCenY() + g.getAlto()/2;
        
		// Determinamos si ha habido colisin (desde el punto de vista de la pelota)
        
        if (!((rightBloque <= leftPelota) || (leftBloque >= rightPelota))) {
			if ((bottomPelota >= topBloque) && (g.getCenY() < this.getCenY())) {
				if ((this.tipoBloque == TIPO_BLOQUE_U)
						|| (this.tipoBloque == TIPO_BLOQUE_LU)
						|| (this.tipoBloque == TIPO_BLOQUE_LRU)
						|| (this.tipoBloque == TIPO_BLOQUE_LRUD)
						|| (this.tipoBloque == TIPO_BLOQUE_LUD)
						|| (this.tipoBloque == TIPO_BLOQUE_RU)
						|| (this.tipoBloque == TIPO_BLOQUE_RUD)
						|| (this.tipoBloque == TIPO_BLOQUE_UD)) {
					// La parte inferior de la pelota colisiona con el bloque
					g.setColisionY(Grafico.COLISION_ABAJO);
				}
			} else if ((topPelota <= bottomBloque)
					&& (g.getCenY() > this.getCenY())) {
				if ((this.tipoBloque == TIPO_BLOQUE_D)
						|| (this.tipoBloque == TIPO_BLOQUE_LD)
						|| (this.tipoBloque == TIPO_BLOQUE_LRD)
						|| (this.tipoBloque == TIPO_BLOQUE_LRUD)
						|| (this.tipoBloque == TIPO_BLOQUE_LUD)
						|| (this.tipoBloque == TIPO_BLOQUE_RD)
						|| (this.tipoBloque == TIPO_BLOQUE_RUD)
						|| (this.tipoBloque == TIPO_BLOQUE_UD)) {
					// La parte superior de la pelota colisiona con el bloque
					g.setColisionY(Grafico.COLISION_ARRIBA);
				}
			}
		}
        
        	
		if (!((bottomBloque <= topPelota) || (topBloque >= bottomPelota))) {
			if ((rightPelota >= leftBloque) && (g.getCenX() < this.getCenX())) {
				if ((this.tipoBloque == TIPO_BLOQUE_L)
						|| (this.tipoBloque == TIPO_BLOQUE_LR)
						|| (this.tipoBloque == TIPO_BLOQUE_LU)
						|| (this.tipoBloque == TIPO_BLOQUE_LD)
						|| (this.tipoBloque == TIPO_BLOQUE_LRU)
						|| (this.tipoBloque == TIPO_BLOQUE_LRD)
						|| (this.tipoBloque == TIPO_BLOQUE_LRUD)
						|| (this.tipoBloque == TIPO_BLOQUE_LUD)) {
					// La parte derecha de la pelota colisiona con el bloque
					g.setColisionX(Grafico.COLISION_DERECHA);
				}
			} else if ((leftPelota <= rightBloque)
					&& (g.getCenX() > this.getCenX())) {
				if ((this.tipoBloque == TIPO_BLOQUE_R)
						|| (this.tipoBloque == TIPO_BLOQUE_LR)
						|| (this.tipoBloque == TIPO_BLOQUE_LRU)
						|| (this.tipoBloque == TIPO_BLOQUE_LRD)
						|| (this.tipoBloque == TIPO_BLOQUE_LRUD)
						|| (this.tipoBloque == TIPO_BLOQUE_RU)
						|| (this.tipoBloque == TIPO_BLOQUE_RD)
						|| (this.tipoBloque == TIPO_BLOQUE_RUD)) {
					// La parte izquierda de la pelota colisiona con el bloque
					g.setColisionX(Grafico.COLISION_IZQUIERDA);
				}
			}
		}                      
    }

	// Fijamos el tipo de bloque
    public void setTipoBloque(int tipoBloque) {
		this.tipoBloque = tipoBloque;
	}       
}
