package com.example.gasullajproyectomaster;

import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.view.View;

class Grafico {

    protected Drawable drawable;		// Imagen que dibujaremos
    protected int cenX, cenY; 			// Posicin del centro del grfico
    protected int ancho, alto; 			// Dimensiones de la imagen
    protected double incX, incY; 		// Velocidad desplazamiento
    private double angulo;				// ngulo de rotacin   
    private double rotacion;			// Velocidad de rotacin
    protected int radioColision; 		// Para determinar colisin
    protected int xAnterior, yAnterior; // Posicin anterior
    protected int radioInval; 			// Radio usado en invalidate()
    protected View view; 				// Usada en view.invalidate()
    private int colisionX;				// Indica si ha colisionado en el eje X
    private int colisionY;				// Indica si ha colisionado en el eje Y
    public static final double VELOC_ROTACION = 3.0d; // Velocidad de rotacin
    
    // Indican si algn lado del grfico ha colisionado con otro objeto.
    // Se asignan a las variables colisionX y colisionY
    public static final int COLISION_NO      	= 0;
    public static final int COLISION_ARRIBA		= 1;
    public static final int COLISION_ABAJO     	= 2;
    public static final int COLISION_IZQUIERDA	= 3;
    public static final int COLISION_DERECHA 	= 4;

    public Grafico(View view, Drawable drawable) {
        this.view     	= view;
        this.drawable	= drawable;
        ancho 			= drawable.getIntrinsicWidth();
        alto  			= drawable.getIntrinsicHeight();
        angulo 			= 0.0d;
        radioColision 	= (alto + ancho) / 4;
        radioInval  	= (int) Math.hypot(ancho / 2, alto / 2);
        rotacion 		= VELOC_ROTACION;
    }

    public void dibujaGrafico(Canvas canvas) {
        int x = cenX - ancho / 2;
        int y = cenY - alto / 2;
        drawable.setBounds(x, y, x + ancho, y + alto);
        canvas.save();
        canvas.rotate((float) angulo, cenX, cenY);
        drawable.draw(canvas);
        canvas.restore();
        view.invalidate(cenX - radioInval, cenY - radioInval,
                cenX + radioInval, cenY + radioInval);
        view.invalidate(xAnterior - radioInval, yAnterior - radioInval,
                xAnterior + radioInval, yAnterior + radioInval);
        xAnterior = cenX;
        yAnterior = cenY;
    }

    public void incrementaPos(double factor) {
        cenX += incX * factor;
        cenY += incY * factor;        
        angulo += rotacion * factor;
    }
   
    public double distancia(Grafico g) {
        return Math.hypot(cenX - g.cenX, cenY - g.cenY);
    }

    public boolean verificaColision(Grafico g) {
        return (distancia(g) < (radioColision + g.radioColision));
    }
    
    public Drawable getDrawable() {
        return drawable;
    }

    public void setDrawable(Drawable drawable) {
        this.drawable = drawable;
    }

    public int getCenX() {
        return cenX;
    }

    public void setCenX(int cenX) {
        this.cenX = cenX;
    }

    public int getCenY() {
        return cenY;
    }

    public void setCenY(int cenY) {
        this.cenY = cenY;
    }

    public int getAncho() {
        return ancho;
    }

    public void setAncho(int ancho) {
        this.ancho = ancho;
    }

    public int getAlto() {
        return alto;
    }

    public void setAlto(int alto) {
        this.alto = alto;
    }

    public double getIncX() {
        return incX;
    }

    public void setIncX(double incX) {
        this.incX = incX;
    }

    public double getIncY() {
        return incY;
    }

    public void setIncY(double incY) {
        this.incY = incY;
    }
    
    public double getAngulo() {
		return angulo;
	}

	public void setAngulo(double angulo) {
		this.angulo = angulo;
	}

	public double getRotacion() {
		return rotacion;
	}

	public void setRotacion(double rotacion) {
		this.rotacion = rotacion;
	}


    public int getRadioColision() {
        return radioColision;
    }

    public void setRadioColision(int radioColision) {
        this.radioColision = radioColision;
    }

    public int getxAnterior() {
        return xAnterior;
    }

    public void setxAnterior(int xAnterior) {
        this.xAnterior = xAnterior;
    }

    public int getyAnterior() {
        return yAnterior;
    }

    public void setyAnterior(int yAnterior) {
        this.yAnterior = yAnterior;
    }

    public int getRadioInval() {
        return radioInval;
    }

    public void setRadioInval(int radioInval) {
        this.radioInval = radioInval;
    }

    public View getView() {
        return view;
    }

    public void setView(View view) {
        this.view = view;
    }

    public int getColisionX() {
        return colisionX;
    }

    public void setColisionX(int colisionX) {
        this.colisionX = colisionX;
    }

    public int getColisionY() {
        return colisionY;
    }

    public void setColisionY(int colisionY) {
        this.colisionY = colisionY;
    }

}
