package com.example.gasullajproyectomaster;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import org.opencv.android.BaseLoaderCallback;
import org.opencv.android.CameraBridgeViewBase;
import org.opencv.android.CameraBridgeViewBase.CvCameraViewFrame;
import org.opencv.android.CameraBridgeViewBase.CvCameraViewListener2;
import org.opencv.android.LoaderCallbackInterface;
import org.opencv.android.OpenCVLoader;
import org.opencv.core.Core;
import org.opencv.core.Mat;
import org.opencv.core.MatOfRect;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.core.Scalar;
import org.opencv.core.Size;
import org.opencv.objdetect.CascadeClassifier;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.TextView;

public class JuegoyFaceDetect extends Activity implements CvCameraViewListener2 {
    private static final String TAG	= "JuegoyFaceDetect";
    private static final Scalar FACE_RECT_COLOR     	= new Scalar(0, 255, 0, 255);
    private static final Scalar FACE_INICIAL_RECT_COLOR = new Scalar(0, 0, 255, 255);
    public static final int JAVA_DETECTOR	= 0;
    public static final int NATIVE_DETECTOR	= 1;

    private Mat mRgba;
    private Mat mGray;
    
    private File mCascadeFile; // Fichero utilizado para realizar la deteccin de caras

    private CascadeClassifier 	  mJavaDetector;    // Deteccin de caras basada en cdigo Java
    private DetectionBasedTracker mNativeDetector;  // Deteccin de caras basada en cdigo nativo

    private int mDetectorType; // Tipo de deteccin (Java o nativa)

    private float mRelativeFaceSize; // Tamanyo mnimo relativo de cara en porcentaje sobre cada eje
    private int   mAbsoluteFaceSize; // Tamanyo total de cara    

    private CameraBridgeViewBase mOpenCvCameraView;
    
    private Procesador procesador; // Utilizada para reflejar la imagen de la cara horiz., usando cdigo nativo

    private VistaJuego vistaJuego; // Vista con el laberinto

    private Point puntoReferencia = new Point(-1, -1); 	// Centro de la referencia para el movimiento 
    private Point centroFace   = new Point(); 			// Centro de la cara detectada de mayor tamanyo
    
    private static final double ZONA_QUIETA = 0.04; // En funcin de la resolucin establece las zonas de movimiento
    private static int VarMinX; // Tamanyo de la zona sin movimiento en el eje X
    private static int VarMinY; // Tamanyo de la zona sin movimiento en el eje Y
    
    private static boolean iniciar; // True movemos la pelota, false la detenemos. Se comprueba en VistaJuego
    
    private TextView textViewModoManejo;
    private Button buttonInicio, buttonSalir;
    
    private SharedPreferences pref;
    private int manejo; // Indica como manejar la pelota (deteccin de cara, pantalla tctil o sensor de acel)
    
    private BaseLoaderCallback  mLoaderCallback = new BaseLoaderCallback(this) {
        @Override
        public void onManagerConnected(int status) {
            switch (status) {
                case LoaderCallbackInterface.SUCCESS:
                {
                    Log.i(TAG, "OpenCV loaded successfully");

                    // Cargamos las libreras despus de la inicializacin de OpenCV
                    System.loadLibrary("detection_based_tracker");
                    System.loadLibrary("mi_libreria_opencv"); // Refleja horiz. la imagen

                    try {                    	                    	
						// Cargamos el fichero lbpcascade para deteccin frontal de cara
						InputStream is = getResources().openRawResource(R.raw.lbpcascade_frontalface);
						File cascadeDir = getDir("cascade", Context.MODE_PRIVATE);
						mCascadeFile = new File(cascadeDir, "lbpcascade_frontalface.xml");
						FileOutputStream os = new FileOutputStream(mCascadeFile);

						byte[] buffer = new byte[4096];
						int bytesRead;
						while ((bytesRead = is.read(buffer)) != -1) {
							os.write(buffer, 0, bytesRead);
						}
						is.close();
						os.close();

						// Deteccin mediante cdigo Java
						mJavaDetector = new CascadeClassifier(
								mCascadeFile.getAbsolutePath());
						if (mJavaDetector.empty()) {
							Log.e(TAG, "Failed to load cascade classifier");
							mJavaDetector = null;
						} else
							Log.i(TAG, "Loaded cascade classifier from "
									+ mCascadeFile.getAbsolutePath());

						// Deteccin mediante cdigo nativo
						mNativeDetector = new DetectionBasedTracker(
								mCascadeFile.getAbsolutePath(), 0);

						cascadeDir.delete();						
                    } catch (IOException e) {
                        e.printStackTrace();
                        Log.e(TAG, "Error al cargar el fichero. Exception: " + e);
                    }                    

                    mOpenCvCameraView.enableView();
                    
                 	// Fijamos el tipo de deteccin al valor definido en preferencias
                    setDetectorType();
                                                     
                } break;
                default:
                {
                    super.onManagerConnected(status);
                } break;
            }
        }
    };    

    public JuegoyFaceDetect() {
    	iniciar = false;
    	      						                
        Log.i(TAG, "Nueva instancia: " + this.getClass());
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        Log.i(TAG, "onCreate");
        super.onCreate(savedInstanceState);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        
        setContentView(R.layout.juego);
        
        textViewModoManejo = (TextView) findViewById(R.id.tv_modo_manejo);
        buttonInicio = (Button) findViewById(R.id.b_inicio);
        buttonSalir  = (Button) findViewById(R.id.b_salir);
        
        vistaJuego = (VistaJuego) findViewById(R.id.VistaJuego);
        vistaJuego.setPadre(this); // Indicamos a vistaJuego que somos su padre

        mOpenCvCameraView = (CameraBridgeViewBase) findViewById(R.id.camera_view);
        mOpenCvCameraView.setCvCameraViewListener(this);
        
        // Establecemos la cmara frontal
        mOpenCvCameraView.setCameraIndex(1);
                                     
        // Listener onClick del botn inicio        
     	buttonInicio.setOnClickListener(new OnClickListener() {
     		public void onClick(View view) {
     			// Si estamos jugando
     			if (iniciar) {
     				// Indicamos que pausamos el juego
     				iniciar = false;
     				// Fijamos el texto para reanudar la partida
     				buttonInicio.setText(getString(R.string.iniciar));
     			} else {
     				iniciar = true;
     				// Fijamos el texto para pausar la partida
     				buttonInicio.setText(getString(R.string.pausar));
     			}     									
     		}
     	});
     					
		// Listener onClick del botn salir        
     	buttonSalir.setOnClickListener(new OnClickListener() {
     		public void onClick(View view) {
     			// Salimos del juego
     			vistaJuego.salir();
     		}
     	});
     	
     	// Obtenemos las preferencias
     	pref = this.getSharedPreferences(
     			getPackageName() + "_preferences",
     			Context.MODE_PRIVATE);	
     		
     	// Obtenemos el valor que nos indicar como manejar la pelota
     	manejo = Integer.valueOf(pref.getString("manejo", Integer.toString(VistaJuego.MANEJO_ROSTRO)));
     	
        // Establecemos el texto que indica como estamos manejando el juego
     	String modoManejo = "";
     	switch (manejo) {
     		case VistaJuego.MANEJO_ROSTRO:     			
     			modoManejo = getString(R.string.manejoRostro);
     			break;
     		case VistaJuego.MANEJO_TACTIL:
     			modoManejo = getString(R.string.manejoTactil);
     			break;     		
     		case VistaJuego.MANEJO_SENSOR_ACELER:
     			modoManejo = getString(R.string.manejoSensorAceler);
     			break;
     	}
        textViewModoManejo.setText(modoManejo);
     	
		// Obtenemos el valor que indica el tamao mnimo de cara en cada dimensin
		int tamanyoMinCara = Integer.valueOf(pref.getString("tamanyoMinCara", "0"));
		
		// Fijamos el tamao mnimo de cara al valor obtenido
     	setMinFaceSize(tamanyoMinCara);    	   	    	
    }

    @Override
    public void onPause()
    {
        super.onPause();
        if (mOpenCvCameraView != null) {
            mOpenCvCameraView.disableView();
        }
                
        // Desactivamos los sensores de aceleracin
        vistaJuego.desactivarSensores();

        // Pausamos el thread del juego
        vistaJuego.getThread().pausar();
    }

    @Override
    public void onResume()
    {
        super.onResume();
        
        OpenCVLoader.initAsync(OpenCVLoader.OPENCV_VERSION_2_4_10, this, mLoaderCallback);
               
        // Reanudamos el thread del juego
        vistaJuego.getThread().reanudar();
        
        // Activamos sensores en funcin de las preferencias
     	vistaJuego.activarSensores();     	     	
    }

    public void onDestroy() {
    	// Detenemos el thread del juego
    	vistaJuego.getThread().detener();
        
    	// Pausamos el juego
    	iniciar = false;
    	
    	super.onDestroy();
    	
        mOpenCvCameraView.disableView();        
    }
    
    public void onCameraViewStarted(int width, int height) {
        mGray = new Mat();     
        mRgba = new Mat();
        
        // Fijamos las variables que establecen las zonas de 
        // movimiento, en funcin de la resolucin de la cmara
        VarMinX = (int) (ZONA_QUIETA * width);
        VarMinY = VarMinX;
                
        // Aqu ya se ha cargado la librera OpenCV
        procesador = new Procesador();
    }

    public void onCameraViewStopped() {
        mGray.release();
        mRgba.release();
    }

    public Mat onCameraFrame(CvCameraViewFrame inputFrame) {   	
    	mRgba = inputFrame.rgba();
    	 	
    	// Si no estamos utilizando deteccin de cara
        if (manejo != VistaJuego.MANEJO_ROSTRO) {
        	// Fijamos una matriz de ceros
            mRgba = Mat.zeros(inputFrame.rgba().size(), inputFrame.rgba().type());
            
            // Dibujamos una x
            Core.line(mRgba, new Point(0, 0), new Point(mRgba.cols(), mRgba.rows()), FACE_RECT_COLOR, 3);
            Core.line(mRgba, new Point(mRgba.cols(), 0), new Point(0, mRgba.rows()), FACE_RECT_COLOR, 3);
            
        	return mRgba;
        }
       
        mGray = inputFrame.gray();
        
        // Si todava no hemos fijado el punto de referencia
        if (puntoReferencia.x == -1) {
        	// Fijamos el punto de referencia en el centro de la pantalla        	
        	puntoReferencia.x = (int)(mRgba.width() / 2);
        	puntoReferencia.y = (int)(mRgba.height() / 2);        	
        }
                       
        // Fijamos el tamanyo mnimo de cara
        if (mAbsoluteFaceSize == 0) {
            int height = mGray.rows();
            if (Math.round(height * mRelativeFaceSize) > 0) {
                mAbsoluteFaceSize = Math.round(height * mRelativeFaceSize);
            }
            mNativeDetector.setMinFaceSize(mAbsoluteFaceSize);
        }

        // Contendr las caras detectadas
        MatOfRect faces = new MatOfRect();
     
		// Procedemos a detectar las caras utilizando el modo de
        // deteccin seleccionado (Java o nativo)
        if (mDetectorType == JAVA_DETECTOR) {
			if (mJavaDetector != null) {
				mJavaDetector.detectMultiScale(mGray, faces, 1.1, 2,
						2, // TODO: objdetect.CV_HAAR_SCALE_IMAGE
						new Size(mAbsoluteFaceSize, mAbsoluteFaceSize),
						new Size());
			}
		} else if (mDetectorType == NATIVE_DETECTOR) {
			if (mNativeDetector != null) {
				mNativeDetector.detect(mGray, faces);
			}
		} else {
			Log.e(TAG, "Mtodo de deteccin no seleccionado");
		}
         
        // Array que contiene las caras detectadas
		Rect[] facesArray = faces.toArray();
                          
        // Nos quedamos nicamente con la cara ms grande de entre todas las detectadas
        int iMax = -1;
        double areaMax = 0;
        for (int i = 0; i < facesArray.length; i++) {
            if (facesArray[i].area() > areaMax) {
                // ndice del array correspondiente a la cara mayor
            	iMax = i;
            	// Area mayor
                areaMax = facesArray[i].area();
            }
        }

        // Si se ha detectado una cara
        if (iMax != -1) {                           	
            // Dibujamos un rectngulo donde se ha detectado la cara
        	Core.rectangle(mRgba, facesArray[iMax].tl(), facesArray[iMax].br(), FACE_RECT_COLOR, 3);
            centroFace.x = facesArray[iMax].x +facesArray[iMax].width/2;
            centroFace.y = facesArray[iMax].y +facesArray[iMax].height/2;
            // Dibujamos un pequeo crculo en el centro del rectngulo
            Core.circle(mRgba, centroFace, 3, FACE_RECT_COLOR, 3);            
            // Procesamos el rostro elegido
            procesa(facesArray[iMax]);          
        } else {
        	// Comento las lneas siguientes, para que si no detecta una cara mantenga
        	// el ltimo movimiento, as evito pequenyas interrupciones al mover la pelota
        	
            //VistaJuego.setMovimientoX(VistaJuego.QUIETO);
            //VistaJuego.setMovimientoY(VistaJuego.QUIETO);
        }

        // Dibujamos la referencia inicial para definir el movimiento
        if (puntoReferencia.x != -1) {
            Core.line(mRgba, new Point(0, (int)puntoReferencia.y - VarMinY), 
            		new Point(mRgba.cols(), (int)puntoReferencia.y - VarMinY), FACE_INICIAL_RECT_COLOR, 3);
            Core.line(mRgba, new Point(0, (int)puntoReferencia.y + VarMinY), 
            		new Point(mRgba.cols(), (int)puntoReferencia.y + VarMinY), FACE_INICIAL_RECT_COLOR, 3);
            Core.line(mRgba, new Point((int)puntoReferencia.x - VarMinX, 0), 
            		new Point((int)puntoReferencia.x - VarMinX, mRgba.rows()), FACE_INICIAL_RECT_COLOR, 3);
            Core.line(mRgba, new Point((int)puntoReferencia.x + VarMinX, 0), 
            		new Point((int)puntoReferencia.x + VarMinX, mRgba.rows()), FACE_INICIAL_RECT_COLOR, 3);
        }

        // Reflejamos la imagen de derecha a izquierda
        mRgba = procesador.ReflejaHoriz(mRgba);

        // Llamamos al garbage collector
        System.gc();
                
        return mRgba;
    }
    
    // Determina el movimiento que debe tener la pelota,
    // en funcin de la posicin de la cara detectada
    public void procesa(Rect face) {     	    	
    	// Si hemos pausado el juego
    	if (!iniciar) {
    		// Detenemos la pelota
    		VistaJuego.setMovimientoX(VistaJuego.QUIETO);
    		VistaJuego.setMovimientoY(VistaJuego.QUIETO);
    		// Retornamos
    		return;
    	}    	    	
    	
        // Si la cara detectada est a la izquierda de la referencia
    	if ((face.x+face.width/2 - puntoReferencia.x) > VarMinX) {
    		// Indicamos que la pelota debe moverse a la izquierda
            VistaJuego.setMovimientoX(VistaJuego.IZQUIERDA);
        // Si la cara detectada est a la derecha de la referencia   
        } else if ((puntoReferencia.x - (face.x+face.width/2)) > VarMinX) {
        	// Indicamos que la pelota debe moverse a la derecha
            VistaJuego.setMovimientoX(VistaJuego.DERECHA);
        } else {
        	// Indicamos que la pelota no debe moverse en el eje x
            VistaJuego.setMovimientoX(VistaJuego.QUIETO);
        }

    	// Si la cara detectada est por debajo de la referencia
    	if ((face.y+face.height/2 - puntoReferencia.y) > VarMinY) {
    		// Indicamos que la pelota debe moverse hacia abajo
            VistaJuego.setMovimientoY(VistaJuego.ABAJO);
        // Si la cara detectada est por encima de la referencia
        } else if ((puntoReferencia.y - (face.y+face.height/2)) > VarMinY) {
        	// Indicamos que la pelota debe moverse hacia arriba
            VistaJuego.setMovimientoY(VistaJuego.ARRIBA);
        } else {
        	// Indicamos que la pelota no debe moverse en el eje y
            VistaJuego.setMovimientoY(VistaJuego.QUIETO);
        }       
    }
        
    // Establecemos el tamanyo mnimo de cara a detectar,
    // expresado en porcentaje en cada dimensin
    private void setMinFaceSize(int faceSize) {
    	// Si hemos elegido el tipo pequeno
    	if (faceSize == 0) {
    		// Establecemos el tamao pequeo
    		mRelativeFaceSize = Float.valueOf(getResources().getString(R.string.tamanyoMinCaraPequenya)) / 100;    				
    	} else {
    		// Establecemos el tamao grande
    		mRelativeFaceSize = Float.valueOf(getResources().getString(R.string.tamanyoMinCaraGrande)) / 100;
    	}
    	     
        mAbsoluteFaceSize = 0;
    }

    // Fijamos el tipo de deteccin (Java o nativa)
    private void setDetectorType() {

    	// Fijamos el tipo de detector al valor obtenido de preferencias (el valor por defecto es Java)
    	mDetectorType = Integer.valueOf(pref.getString("tipoDeteccionCara", Integer.toString(JAVA_DETECTOR)));

        // Si se ha elegido deteccin nativa
        if (mDetectorType == NATIVE_DETECTOR) {
        	Log.i(TAG, "Detection Based Tracker enabled");
            // Activamos la deteccin nativa
            mNativeDetector.start();                
        } else {
            Log.i(TAG, "Cascade detector enabled");
        }
    }

	public static boolean isIniciar() {
		return iniciar;
	}	    
}
