package com.example.gasullajproyectomaster;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.NotificationManager;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.drawable.Drawable;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.support.v4.app.NotificationCompat;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewTreeObserver.OnGlobalLayoutListener;
import android.widget.ImageButton;
import android.widget.Toast;

public class MainActivity extends Activity {

	private ImageButton ibJugar, ibConfigurar, ibPuntuaciones, ibInfo,
			ibAcercaDe;	
	private View mainView; // Contenedor principal
	int mainWidth; 	// Contendr la anchura de la pantalla
	int mainHeight;	// Contendr la altura de la pantalla
	private SharedPreferences pref; 
	public static AlmacenPuntuaciones almacen;
	private MediaPlayer mp;
	private static final int ID_NOTIFICACION_CREAR = 1;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		// Obtenemos las preferencias
		pref = this.getSharedPreferences(getPackageName() + "_preferences",
				Context.MODE_PRIVATE);
			
		// Msica de fondo
		mp = MediaPlayer.create(this, R.raw.musica_fondo);
		// La msica se repetir en bucle
		mp.setLooping(true);
		
		// ImageButtons de la pantalla principal
		ibJugar 		= (ImageButton) findViewById(R.id.imagebutton_play);
		ibConfigurar 	= (ImageButton) findViewById(R.id.imagebutton_configurar);
		ibAcercaDe 		= (ImageButton) findViewById(R.id.imagebutton_acercaDe);
		ibInfo 			= (ImageButton) findViewById(R.id.imagebutton_info);
		ibPuntuaciones 	= (ImageButton) findViewById(R.id.imagebutton_puntuaciones);

		// Layout contenedor de la pantalla principal
		mainView = (View) findViewById(R.id.layoutContenedorMain);

		mainView.getViewTreeObserver().addOnGlobalLayoutListener(
				new OnGlobalLayoutListener() {
					@SuppressLint("NewApi")
					@SuppressWarnings("deprecation")
					@Override
					public void onGlobalLayout() {
						// Obtenemos las dimensiones del contenedor
						mainWidth = mainView.getWidth();
						mainHeight = mainView.getHeight();

						// Ajustamos el tamao de los elementos
						// proporcionalmente al contenedor
						ibJugar.setBackground(AjustesDrawables.ajustarDrawable(
								getBaseContext(), R.drawable.play, 0.5d,
								mainWidth, mainHeight));
						ibConfigurar.setBackground(AjustesDrawables
								.ajustarDrawable(getBaseContext(),
										R.drawable.settings, 0.20d, mainWidth,
										mainHeight));												
						ibAcercaDe.setBackground(AjustesDrawables
								.ajustarDrawable(getBaseContext(),
										R.drawable.acercade, 0.15d, mainWidth,
										mainHeight));
						ibInfo.setBackground(AjustesDrawables.ajustarDrawable(
								getBaseContext(), R.drawable.info, 0.15d,
								mainWidth, mainHeight));
						ibPuntuaciones.setBackground(AjustesDrawables
								.ajustarDrawable(getBaseContext(),
										R.drawable.scores, 0.15d, mainWidth,
										mainHeight));

						// Eliminamos el listener, en funcin del nivel de API
						if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.JELLY_BEAN) {
							mainView.getViewTreeObserver()
									.removeOnGlobalLayoutListener(this);
						} else {
							mainView.getViewTreeObserver()
									.removeGlobalOnLayoutListener(this);
						}
					}
				});

		// Listener onClick del botn Jugar
		ibJugar.setOnClickListener(new OnClickListener() {
			public void onClick(View view) {
				// Lanzamos el juego
				lanzarJuego(null);
			}
		});

		// Listener onClick del botn Configurar
		ibConfigurar.setOnClickListener(new OnClickListener() {
			public void onClick(View view) {
				// Lanzamos la pantalla con las preferencias
				lanzarPreferencias(null);
			}
		});

		// Listener onClick del botn Acerca de
		ibAcercaDe.setOnClickListener(new OnClickListener() {
			public void onClick(View view) {
				// Lanzamos la pantalla acercaDe
				lanzarAcercaDe(null);
			}
		});

		// Listener onClick del botn Info
		ibInfo.setOnClickListener(new OnClickListener() {
			public void onClick(View view) {
				// Lanzamos la pantalla de informacin
				lanzarInfo(null);
			}
		});

		// Listener onClick del botn Puntuaciones
		ibPuntuaciones.setOnClickListener(new OnClickListener() {
			public void onClick(View view) {
				// Lanzamos la pantalla con las puntuaciones 
				lanzarPuntuaciones(null);
			}
		});

		// Almacn con las puntuaciones
		almacen = new AlmacenPuntuacionesSQLite(this);
	}

	@Override
	protected void onResume() {
		super.onResume();
		
		// Si hemos activado la msica, y se ha creado correctamente
		if (pref.getBoolean("musica", true) && mp != null) {						
			// Reproducimos la msica cuando la actividad est visible
			mp.start();			
		}
	}

	@Override
	protected void onPause() {
		// Si hemos activado la msica, y se ha creado correctamente
		if (pref.getBoolean("musica", true) && mp != null) {			
			// Para la msica cuando la actividad ya no est activa
			mp.pause();			
		}
		super.onPause();
	}

	// Lanzamos el juego
	public void lanzarJuego(View view) {
		Intent i = new Intent(this, JuegoyFaceDetect.class);
		startActivityForResult(i, 1234);
	}

	// Lanzamos la pantalla con las preferencias
	public void lanzarPreferencias(View view) {
		Intent i = new Intent(this, Preferencias.class);
		startActivity(i);
	}

	// Lanzamos la pantalla con las puntuaciones
	public void lanzarPuntuaciones(View view) {
		Intent i = new Intent(this, Puntuaciones.class);
		startActivity(i);
	}

	// Lanzamos la pantalla acercaDe
	public void lanzarAcercaDe(View view) {
		Intent i = new Intent(this, AcercaDe.class);
		startActivity(i);
	}

	// Lanzamos la pantalla de informacin
	public void lanzarInfo(View view) {
		Intent i = new Intent(this, Instrucciones.class);
		startActivity(i);
	}

	// Componemos un texto con la informacin de la partida
	public String componerTexto(int puntuacion, String nombre) {
		String texto = getString(R.string.jugador) + ": " + nombre + ", "
				+ getString(R.string.puntuacion) + ": " + puntuacion;
		return texto;
	}

	// Lanzamos una notificacin con la informacin de la partida
	public void lanzarNotificacion(String title, String text) {
		NotificationCompat.Builder notific = new NotificationCompat.Builder(
				this).setContentTitle(title)
				.setSmallIcon(R.drawable.ic_launcher).setContentText(text);
		NotificationManager notificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
		notificationManager.notify(ID_NOTIFICACION_CREAR, notific.build());
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == 1234 && resultCode == RESULT_OK && data != null) {

			// Recuperamos la puntuacin obtenida
			int puntuacion = data.getExtras().getInt("puntuacion");

			// Obtenemos el nombre desde las preferencias
			String nombre = pref.getString("nombreJugador",
					getString(R.string.yo));

			// Si el nombre est vaco
			if (nombre.trim().length() == 0) {
				// Ponemos el nombre por defecto
				nombre = getString(R.string.yo);
			}
			
			// Guardamos la puntuacin en la base de datos
			almacen.guardarPuntuacion(puntuacion, nombre,
					System.currentTimeMillis());

			// Lanzaremos una notificacin con los datos 
			// relativos a la partida que acaba de jugar
			String asunto = getString(R.string.app_name);
			String texto = componerTexto(puntuacion, nombre);

			// Lanzamos la notificacin
			lanzarNotificacion(asunto, texto);

			// Mostramos los mismos datos en un Toast
			Toast.makeText(this, texto,Toast.LENGTH_SHORT).show();
		}
	}

	@Override
	protected void onSaveInstanceState(Bundle estadoGuardado) {
		super.onSaveInstanceState(estadoGuardado);
		// Si hemos activado la msica y se  
		// ha creado el objeto MediaPlayer
		if (mp != null && pref.getBoolean("musica", true)) {
			// Guardamos la posicin de reproduccin
			int pos = mp.getCurrentPosition();
			estadoGuardado.putInt("posicion", pos);
		}
	}

	@Override
	protected void onRestoreInstanceState(Bundle estadoGuardado) {
		super.onRestoreInstanceState(estadoGuardado);
		// Si hemos activado la msica y tenemos una posicin guardada
		if (estadoGuardado != null && mp != null
				&& pref.getBoolean("musica", true)) {
			// Recuperamos la posicin de reproduccin
			int pos = estadoGuardado.getInt("posicion");
			// Continuamos la reproduccin en la posicin recuperada
			mp.seekTo(pos);
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.itemInfo:
			lanzarInfo(null);
			break;

		case R.id.itemConfig:
			lanzarPreferencias(null);
			break;

		case R.id.itemPuntos:
			lanzarPuntuaciones(null);
			break;
		}

		return super.onOptionsItemSelected(item);
	}
}
