package com.example.gasullajproyectomaster;

import java.util.List;
import java.util.Vector;

import android.annotation.TargetApi;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.ShapeDrawable;
import android.graphics.drawable.shapes.PathShape;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.media.AudioAttributes;
import android.media.AudioManager;
import android.media.SoundPool;
import android.os.Build;
import android.os.Bundle;
import android.os.Vibrator;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Toast;

public class VistaJuego extends View implements SensorEventListener  {

	private Activity padre;
    private Context context;
	
	////// THREAD Y TIEMPO //////    
    private ThreadJuego thread = new ThreadJuego(); // Thread encargado de procesar el juego    
    private static int PERIODO_PROCESO = 50; 		// Cada cuanto tiempo (ms) se procesan los cambios    
    private long ultimoProceso         = 0; 		// Cuando se realiz el ltimo proceso

    ////// JUEGO //////
    private static int NCOLUMNAS = 12;	// N de columnas en que se divide la pantalla
    private static int NFILAS 	 = 9;  	// N de filas en que se divide la pantalla    
    private static final int NUM_NIVELES = 2; 	// N de niveles del juego
    private int nivel 					 = 1;	// Indica el nivel actual, inicialmente nivel 1     
	private static final double FACTOR_REDUCCION_DRAWABLES = 0.75; // Para ajustar el tamao de los drawables 
    private int anchoPantalla = 0;	// Guardaremos el ancho de la vista
    private int altoPantalla  = 0;	// Guardaremos el alto  de la vista         
    private int puntuacion    = 0;	// Puntuacin del jugador
	private SharedPreferences pref;
	private int manejo; // Indica como manejar la pelota, se lee de preferencias y se le asigna una constante
	public static final int MANEJO_ROSTRO  		 = 0; // La pelota se mueve por deteccin de cara
	public static final int MANEJO_TACTIL  		 = 1; // La pelota se mueve mediante la pantalla tctil
	public static final int MANEJO_SENSOR_ACELER = 2; // La pelota se mueve mediante el sensor de aceleracin
    
    ////// PELOTA ///////
    private Grafico pelota;  					// Grfico de la pelota
    private static final double PASO_VELOCIDAD = 0.00435; // Paso de la velocidad de la pelota 
    private static int VelocidadAvanceX;		// Velocidad de la pelota en el eje X (se fija en onSizeChanged)
    private static int VelocidadAvanceY; 		// Velocidad de la pelota en el eje Y (se fija en onSizeChanged)
    private double velXPelota = 0; 				// Velocidad de la pelota en el eje X (0 o +/-VelocidadAvanceX)
    private double velYPelota = 0;				// Velocidad de la pelota en el eje Y (0 o +/-VelocidadAvanceY)
    public static final int QUIETO    = 0; 		// La pelota no se mueve
    public static final int DERECHA   = 1; 		// La pelota se mueve hacia la derecha
    public static final int IZQUIERDA = 2; 		// La pelota se mueve hacia la izquierda
    public static final int ARRIBA    = 3; 		// La pelota se mueve hacia arriba
    public static final int ABAJO     = 4; 		// La pelota se mueve hacia abajo
    private static int movimientoX = QUIETO; 	// Tipo de movimiento x. Inicialmente quieto
    private static int movimientoY = QUIETO; 	// Tipo de movimiento y. Inicialmente quieto
            
    ////// BLOQUES //////
    private Vector<Bloque> bloques;				// Vector con los bloques
    private Drawable drawableBloque; 			// Drawable del bloque prototipo
    private static final int numBloques = 46; 	// N de bloques de cada nivel
    // ndice en el eje Y de las celdas con los bloques del nivel 1
    private int casillasYBloquesNivel1[] = {0,0,1,1,1,1,1,1,1,2,2,2,2,3,3,3,3,3,3,3,3,3,4,4,4,
            5,5,5,5,5,5,5,6,6,6,7,7,7,7,7,7,7,7,7,8,8};
    // ndice en el eje X de las celdas con los bloques del nivel 1
    private int casillasXBloquesNivel1[] = {0,2,0,2,4,5,6,8,10,0,4,8,10,0,2,4,5,6,7,8,9,10,2,6,
            8,0,2,4,5,6,8,10,2,6,10,0,1,2,4,5,6,8,9,10,4,10};
    // Tipo de cada bloque del nivel 1
    private int tiposBloquesNivel1[] = {Bloque.TIPO_BLOQUE_R, Bloque.TIPO_BLOQUE_LR,
            Bloque.TIPO_BLOQUE_R, 	Bloque.TIPO_BLOQUE_LRD, Bloque.TIPO_BLOQUE_LU,
            Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_RUD, Bloque.TIPO_BLOQUE_LRU,
            Bloque.TIPO_BLOQUE_LRU, Bloque.TIPO_BLOQUE_R, 	Bloque.TIPO_BLOQUE_LR,
            Bloque.TIPO_BLOQUE_LR, 	Bloque.TIPO_BLOQUE_LR, 	Bloque.TIPO_BLOQUE_RD,
            Bloque.TIPO_BLOQUE_LRU, Bloque.TIPO_BLOQUE_LD, 	Bloque.TIPO_BLOQUE_UD,
            Bloque.TIPO_BLOQUE_U, 	Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_N,
            Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_RD, 	Bloque.TIPO_BLOQUE_LR,
            Bloque.TIPO_BLOQUE_LR, 	Bloque.TIPO_BLOQUE_LR, 	Bloque.TIPO_BLOQUE_RUD,
            Bloque.TIPO_BLOQUE_LR, 	Bloque.TIPO_BLOQUE_LUD, Bloque.TIPO_BLOQUE_UD, 
            Bloque.TIPO_BLOQUE_R, 	Bloque.TIPO_BLOQUE_LRD, Bloque.TIPO_BLOQUE_LRU, 
            Bloque.TIPO_BLOQUE_LR, 	Bloque.TIPO_BLOQUE_LR, 	Bloque.TIPO_BLOQUE_LR, 
            Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_RD, 
            Bloque.TIPO_BLOQUE_LU, 	Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_RD, 
            Bloque.TIPO_BLOQUE_LUD, Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_R, 
            Bloque.TIPO_BLOQUE_LR, 	Bloque.TIPO_BLOQUE_LR};        
    // ndice en el eje Y de las celdas con los bloques del nivel 2
    private int casillasYBloquesNivel2[] = {0,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,4,4,4,
    		4,4,5,5,5,5,5,5,5,5,6,6,6,7,7,7,7,7,7,7,7,7,7,8};
    // ndice en el eje X de las celdas con los bloques del nivel 2 
    private int casillasXBloquesNivel2[] = {0,0,2,3,4,5,6,7,8,10,11,6,0,2,4,6,8,10,11,2,
    		4,6,8,10,0,2,3,4,6,7,8,10,0,4,6,0,1,2,4,6,7,8,9,10,11,4};  
    // Tipo de cada bloque del nivel 2
    private int tiposBloquesNivel2[] = {Bloque.TIPO_BLOQUE_R, Bloque.TIPO_BLOQUE_RD,
            Bloque.TIPO_BLOQUE_LUD,	Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_UD,
            Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_U, 	Bloque.TIPO_BLOQUE_UD,
            Bloque.TIPO_BLOQUE_RUD, Bloque.TIPO_BLOQUE_LUD, Bloque.TIPO_BLOQUE_UD,
            Bloque.TIPO_BLOQUE_LRD, Bloque.TIPO_BLOQUE_RUD, Bloque.TIPO_BLOQUE_LRU,
            Bloque.TIPO_BLOQUE_LRU, Bloque.TIPO_BLOQUE_LR, 	Bloque.TIPO_BLOQUE_LRU,
            Bloque.TIPO_BLOQUE_LU, 	Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_LR,
            Bloque.TIPO_BLOQUE_LR, 	Bloque.TIPO_BLOQUE_LR, 	Bloque.TIPO_BLOQUE_LR,
            Bloque.TIPO_BLOQUE_LR, 	Bloque.TIPO_BLOQUE_RU, 	Bloque.TIPO_BLOQUE_LD,
            Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_R, 	Bloque.TIPO_BLOQUE_L, 
            Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_RD, 	Bloque.TIPO_BLOQUE_LRD, 
            Bloque.TIPO_BLOQUE_R, 	Bloque.TIPO_BLOQUE_LR, 	Bloque.TIPO_BLOQUE_LR, 
            Bloque.TIPO_BLOQUE_D, 	Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_RUD, 
            Bloque.TIPO_BLOQUE_LR, 	Bloque.TIPO_BLOQUE_LD, 	Bloque.TIPO_BLOQUE_UD, 
            Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_UD, 
            Bloque.TIPO_BLOQUE_UD, 	Bloque.TIPO_BLOQUE_LR};
    
	////// MONEDAS //////
    private Vector<Grafico> monedas;				// Vector con las monedas
    private Drawable drawableMoneda;				// Drawable de la moneda prototipo
    private static final int numMonedas = 3;  		// Nmero de monedas en cada nivel
    private int casillasYMonedasNivel1[] = {2,2,4}; // ndice en el eje Y de las celdas con las monedas del nivel 1
    private int casillasXMonedasNivel1[] = {2,5,7}; // ndice en el eje X de las celdas con las monedas del nivel 1
    private int casillasYMonedasNivel2[] = {3,4,8}; // ndice en el eje Y de las celdas con las monedas del nivel 2
    private int casillasXMonedasNivel2[] = {3,7,9}; // ndice en el eje X de las celdas con las monedas del nivel 2
        
    ////// META //////
    private Grafico meta;  				 // Grfico de la meta   
    private int casillaYMetaNivel1 = 8;  // ndice en el eje Y de la celda donde est la meta del nivel 1
    private int casillaXMetaNivel1 = 9;  // ndice en el eje X de la celda donde est la meta del nivel 1
    private int casillaYMetaNivel2 = 4;  // ndice en el eje Y de la celda donde est la meta del nivel 2
    private int casillaXMetaNivel2 = 11; // ndice en el eje X de la celda donde est la meta del nivel 2
        
	////// SENSORES //////
	private SensorManager mSensorManager;
	private Sensor sensor;
       
	////// SONIDOS //////
    SoundPool sounds;
    int idSonidoMeta, idSonidoMoneda; // Sonidos emitidos al llegar a la meta o coger una moneda    	

    public VistaJuego(Context context, AttributeSet attrs) {
        super(context, attrs);
        this.context = context;
                
        // Asignamos el fondo
        this.setBackgroundResource(R.drawable.fondo);
        
		// Obtenemos las preferencias
		pref = context.getSharedPreferences(
				context.getPackageName() + "_preferences",
				Context.MODE_PRIVATE);				

		// Obtenemos el valor que nos indicar como manejar la pelota
		manejo = Integer.valueOf(pref.getString("manejo", Integer.toString(MANEJO_ROSTRO)));
						
		// Obtenemos el SensorManager		
		mSensorManager = (SensorManager) context.getSystemService(Context.SENSOR_SERVICE);
        
        // Activamos los sonidos, si hemos habilitado la msica por preferencias  		
		if (pref.getBoolean("musica", true)) {
			// En funcin de la versin de API usamos los mtodos correctos
			if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
				createNewSoundPool();
			} else {
				createOldSoundPool();
			}
			createOldSoundPool();
			
			// Cargamos los sonidos
			idSonidoMeta   = sounds.load(context, R.raw.sonido_meta, 0);
			idSonidoMoneda = sounds.load(context, R.raw.sonido_moneda, 0); 
		}			
    }

    @Override
    protected void onSizeChanged(int ancho, int alto, int ancho_anter,
                                 int alto_anter) {
        super.onSizeChanged(ancho, alto, ancho_anter, alto_anter);
        
        // Una vez que conocemos nuestro ancho y alto.

        // Guardamos el tamao de la vista
        anchoPantalla = ancho;
        altoPantalla  = alto;        
        		      		
        // Parte relacionada con la pelota
                
        // Fijamos la velocidad de la pelota en funcin del tamao 
        // de la vista y del modo de manejo de la pelota
        if (manejo == MANEJO_ROSTRO) { 
        	VelocidadAvanceX = (int) (PASO_VELOCIDAD * anchoPantalla);
        	VelocidadAvanceY = VelocidadAvanceX;
        } else if ((manejo == MANEJO_TACTIL) || (manejo == MANEJO_SENSOR_ACELER)){    
        	VelocidadAvanceX = (int) (2 * PASO_VELOCIDAD * anchoPantalla);
        	VelocidadAvanceY = VelocidadAvanceX;
        }                
        // Ajustamos el tamao de la pelota
		Drawable drawablePelota = AjustesDrawables.ajustarDrawable(this.context,
				R.drawable.pelota, FACTOR_REDUCCION_DRAWABLES, 
					anchoPantalla / NCOLUMNAS, altoPantalla / NFILAS);   
        // Creamos la pelota
		pelota = new Grafico(this, drawablePelota);
       
        // Parte relacionada con los bloques
        
        // Damos forma y tamao al bloque tipo
        Path pathBloque = new Path();
        pathBloque.addRect (0.0f, 0.0f, 1.0f, 1.0f, Path.Direction.CW);
        ShapeDrawable dBloque = new ShapeDrawable(new PathShape(pathBloque, 1.0f, 1.0f));
        dBloque.getPaint().setColor(Color.GRAY);
        dBloque.getPaint().setStyle(Paint.Style.FILL);
        dBloque.setIntrinsicWidth(anchoPantalla / NCOLUMNAS);
        dBloque.setIntrinsicHeight(altoPantalla / NFILAS);
        drawableBloque = dBloque;
        // Creamos el vector de bloques
        bloques = new Vector<Bloque>();
        // Creamos los bloques
        creaBloques();        
                        
        // Parte relacionada con las monedas   

        // Ajustamos el tamao de la moneda tipo        
		drawableMoneda = AjustesDrawables.ajustarDrawable(this.context,
				R.drawable.moneda, FACTOR_REDUCCION_DRAWABLES, anchoPantalla
						/ NCOLUMNAS, altoPantalla / NFILAS);        				
		// Creamos el vector de monedas
		monedas = new Vector<Grafico>();
		// Creamos las monedas
		creaMonedas();		
                                                    
        // Parte relacionada con la meta    

        // Ajustamos el tamao de la meta
		Drawable drawableMeta = AjustesDrawables.ajustarDrawable(this.context,
				R.drawable.meta, FACTOR_REDUCCION_DRAWABLES*1.2, anchoPantalla
						/ NCOLUMNAS, altoPantalla / NFILAS);
        // Creamos la meta
        meta = new Grafico(this, drawableMeta);
     
     	// Posicionamos los elementos
     	posicionaElementos();
            
     	// Fijamos el tiempo actual
        ultimoProceso = System.currentTimeMillis();
        
        if (!thread.corriendo) {
        	thread.start();
        }
    }

    @Override
    protected synchronized void onDraw(Canvas canvas) {
        super.onDraw(canvas);

        // Dibujamos los bloques
        for (Grafico bloque : bloques) {
            bloque.dibujaGrafico(canvas);
        }
        
        // Dibujamos las monedas
        for (Grafico moneda : monedas) {
        	moneda.dibujaGrafico(canvas);
        }
        
        // Dibujamos la meta
        meta.dibujaGrafico(canvas);

        // Dibujamos la pelota
        pelota.dibujaGrafico(canvas);                
    }

    protected synchronized void actualizaFisica() {
        long ahora = System.currentTimeMillis();

        // Salimos si el perodo de proceso no se ha cumplido.
        if (ultimoProceso + PERIODO_PROCESO > ahora) {
            return; 
        }
        
        // Si hemos pausado el juego
        if (!JuegoyFaceDetect.isIniciar()) {
    		// Detenemos la pelota
    		setMovimientoX(QUIETO);
    		setMovimientoY(QUIETO);
    		// Salimos
    		return;
    	}

        // Para una ejecucin en tiempo real calculamos retardo
        double retardo = (ahora - ultimoProceso) / PERIODO_PROCESO;
        ultimoProceso = ahora; // Para la prxima vez

        // Actuamos en funcin del sentido del 
        // movimiento de la pelota en el eje x
        switch (movimientoX) {
            case QUIETO:
                velXPelota = 0;               
                break;
            case DERECHA:
                // Si la pelota no est a la dcha de la pantalla
                if ((pelota.getCenX() + pelota.getAncho() / 2) < anchoPantalla) {
                    // La pelota avanza hacia la dcha
                    velXPelota = VelocidadAvanceX;
                } else {
                    // La pelota se detiene
                    velXPelota = 0;
                    // Evitamos que la pelota se dibuje fuera de la pantalla
                    pelota.setCenX(anchoPantalla - pelota.getAncho() / 2);
                }
                break;
            case IZQUIERDA:
                // Si la pelota no est no est a la izqda de la pantalla
                if ((pelota.getCenX() - pelota.getAncho() / 2) > 0) {
                    // La pelota avanza hacia la izqda
                    velXPelota = -VelocidadAvanceX;
                } else {
                    // La pelota se detiene horizontalmente
                    velXPelota = 0;
                    // Evitamos que la pelota se dibuje fuera de la pantalla
                    pelota.setCenX(pelota.getAncho() / 2);
                }
                break;
        }

        // Actuamos en funcin del sentido del 
        // movimiento de la pelota en el eje y
        switch (movimientoY) {
            case QUIETO:
                velYPelota = 0;
                break;
            case ARRIBA:
                // Si la pelota no est arriba en la pantalla
                if ((pelota.getCenY() - pelota.getAlto()/2) > 0) {
                    // La pelota avanza hacia arriba
                    velYPelota = -VelocidadAvanceY;

                } else {
                    // La pelota se detiene verticalmente
                    velYPelota = 0;
                    // Evitamos que la pelotase dibuje fuera de la pantalla
                    pelota.setCenY(pelota.getAlto()/2);
                }
                break;
            case ABAJO:
                // Si la pelota no est abajo en la pantalla
                if ((pelota.getCenY() + pelota.getAlto()/2) < altoPantalla) {
                    // La pelota avanza hacia abajo
                    velYPelota = VelocidadAvanceY;
                } else {
                    // La pelota se detiene verticalmente
                    velYPelota = 0;
                    // Evitamos que la pelota se dibuje fuera de la pantalla
                    pelota.setCenY(altoPantalla - pelota.getAlto()/2);
                }
                break;
        }
        
        // Verificamos si hemos llegado a la meta
        verificaColisionPelotaMeta();
        
        // Verificamos si hay colisin de pelota y alguna moneda
     	verificaColisionPelotaMonedas();

        // Verificamos si hay colisin de pelota y bloque y
        // variamos velocidad y posicin de la pelota, si es necesario
        verificaColisionPelotaBloque();

        // Actualizamos velocidad de la pelota
        pelota.setIncX(velXPelota);
        pelota.setIncY(velYPelota);     

        // Fijamos el sentido de la rotacin de la 
        // pelota en funcin del desplazamiento        
        if (velXPelota > 0) {        
        	pelota.setRotacion(Grafico.VELOC_ROTACION);        	
        } else if (velXPelota < 0) {
        	pelota.setRotacion(Grafico.VELOC_ROTACION * (-1.0d));
        } else if (velYPelota > 0) {        
        	pelota.setRotacion(Grafico.VELOC_ROTACION);        	
        } else if (velYPelota < 0) {
        	pelota.setRotacion(Grafico.VELOC_ROTACION * (-1.0d));
        } else {
        	pelota.setRotacion(0.0d);
        }
        
        // Actualizamos la posicin y el ngulo de rotacin de la pelota     
        pelota.incrementaPos(retardo);
    }
    
    // Creamos el SoundPool para versiones de API 21 o superior
    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
	protected void createNewSoundPool() {
		AudioAttributes attributes = new AudioAttributes.Builder()
				.setUsage(AudioAttributes.USAGE_GAME)
				.setContentType(AudioAttributes.CONTENT_TYPE_SONIFICATION)
				.build();
		sounds = new SoundPool.Builder().setAudioAttributes(
				attributes).build();
	}

    // Creamos el SoundPool para versiones inferiores a API 21
	@SuppressWarnings("deprecation")
	protected void createOldSoundPool() {
		sounds = new SoundPool(5, AudioManager.STREAM_MUSIC, 0);
	}

    public static int getMovimientoX() {
        return movimientoX;
    }

    public static void setMovimientoX(int tipoMovimientoX) {
        movimientoX = tipoMovimientoX;
    }

    public static int getMovimientoY() {
        return movimientoY;
    }

    public static void setMovimientoY(int tipoMovimientoY) {
        movimientoY = tipoMovimientoY;
    }
    
    // Creamos los bloques
    private void creaBloques() {
        for (int i = 0; i < numBloques; i++) {
        	// Inicialmente creamos los bloques para el nivel 1,
        	// posteriormente basta con cambiar el tipo del bloque
        	Bloque bloque = new Bloque(this, drawableBloque, tiposBloquesNivel1[i]);
            bloques.add(bloque);
        }
    }
    
    // Creamos las monedas
    private void creaMonedas() {	
        for (int i = 0; i < numMonedas; i++) {
    		Grafico moneda = new Grafico(this, drawableMoneda);			
    		monedas.add(moneda);
    	}
    }
    
    // Eliminamos las monedas
    private void eliminaMonedas() {
    	// Si quedan monedas
    	if (monedas.size() > 0) {
    		// Recorremos las monedas en orden inverso
    	 	for (int i = monedas.size() - 1; i >= 0; i--) {    	 							 				
   	 	 		// Eliminamos la moneda
    	 	 	monedas.remove(i);    	 				 				
    	 	}
    	 }
    }
    
    // Establecemos el tipo de cada bloque,
    // se usa en niveles diferentes al inicial
	private void estableceTiposBloques() {
		for (int i = 0; i < bloques.size(); i++) {
			if (nivel == 2) {
				bloques.elementAt(i).setTipoBloque(tiposBloquesNivel2[i]);				
			}			
		}            		
	}
        
    // Establecemos la posicin de los elementos
    private void posicionaElementos() {
    	
    	// Posicionamos la pelota en la esquina inferior izquierda
    	pelota.setCenX(pelota.getAncho()/2);
    	pelota.setCenY(altoPantalla - pelota.getAlto()/2);
    	
    	// Posicionamos los bloques
    	posicionaBloques();	
    	
    	// Posicionamos las monedas
		posicionaMonedas();
		
		// Posicionamos la meta
		posicionaMeta();
    }
    
    // Establecemos la posicin de los bloques
    private void posicionaBloques() {
        for (int i = 0; i < bloques.size(); i++) {
            Grafico bloque = bloques.elementAt(i);
            if (nivel == 1) {
            	bloque.setCenX(anchoPantalla/NCOLUMNAS * casillasXBloquesNivel1[i] + bloque.getAncho()/2);
                bloque.setCenY(altoPantalla/NFILAS     * casillasYBloquesNivel1[i] + bloque.getAlto()/2);
            } else {
            	bloque.setCenX(anchoPantalla/NCOLUMNAS * casillasXBloquesNivel2[i] + bloque.getAncho()/2);
                bloque.setCenY(altoPantalla/NFILAS     * casillasYBloquesNivel2[i] + bloque.getAlto()/2);
            }
            
        }
    }
    
    // Establecemos la posicin de las monedas en el centro de la celda correspondiente
    private void posicionaMonedas() {
        for (int i = 0; i < monedas.size(); i++) {
            Grafico moneda = monedas.elementAt(i);   
            if (nivel == 1) {
            	moneda.setCenX((int) (anchoPantalla/NCOLUMNAS * (casillasXMonedasNivel1[i] + 0.5)));
                moneda.setCenY((int) (altoPantalla/NFILAS     * (casillasYMonedasNivel1[i] + 0.5)));
            } else {
            	moneda.setCenX((int) (anchoPantalla/NCOLUMNAS * (casillasXMonedasNivel2[i] + 0.5)));
                moneda.setCenY((int) (altoPantalla/NFILAS     * (casillasYMonedasNivel2[i] + 0.5)));
            }           
        }    
    }
    
    // Establecemos la posicin de la meta en el centro de la celda correspondiente
    private void posicionaMeta() {
        if (nivel == 1) {
        	 meta.setCenX((int) (anchoPantalla/NCOLUMNAS * (casillaXMetaNivel1 + 0.5)));
             meta.setCenY((int) (altoPantalla/NFILAS * (casillaYMetaNivel1 + 0.5)));
        } else {
        	 meta.setCenX((int) (anchoPantalla/NCOLUMNAS * (casillaXMetaNivel2 + 0.5)));
             meta.setCenY((int) (altoPantalla/NFILAS * (casillaYMetaNivel2 + 0.5)));
        }         
    }
    
    // Verificamos si hay colisin de pelota y alguna moneda
 	private void verificaColisionPelotaMonedas() {
 		// Si quedan monedas
 		if (monedas.size() > 0) {
 			// Recorremos las monedas en orden inverso
 			for (int i = monedas.size() - 1; i >= 0; i--) {
 				Grafico moneda = monedas.elementAt(i); 				 				
 				// Si ha colisionado con la pelota
 				if (pelota.verificaColision(moneda)) { 	 				
 					// Incrementamos la puntuacin
 					puntuacion += 100; 					
 	 				
 					// Activamos los sonidos, si hemos
 	 				// habilitado la msica por preferencias
 	 				if (pref.getBoolean("musica", true)) {
 	 					// Reproducimos el sonido asociado
 	 					sounds.play(idSonidoMoneda, 1, 1, 0, 0, 1);
 	 				} 	 				
 				
 	 				// Eliminamos la moneda
 	 				monedas.remove(i);
 				} 				
 			}
 		}
 	}	
 	
    // Verificamos si hay colisin de pelota y bloque y
    // variamos velocidad y posicin de la pelota, si es necesario
    private void verificaColisionPelotaBloque() {
        int newY;
        int newX;
        for (Grafico bloque : bloques) {
            // Verificamos si ha colisionado con la pelota
            ((Bloque) bloque).verificaColisionLados(pelota);
                      
            switch (pelota.getColisionY()) {
            	// Si la parte superior de la pelota ha colisionado con el bloque
                case Bloque.COLISION_ARRIBA:
                    // Si la pelota se est desplazando hacia arriba
                	if (velYPelota < 0) {
                		// Detenemos el movimiento vertical
                        velYPelota = 0;
                    }
                	// Obtenemos la nueva posicin de la pelota en el eje y,
                	// para que cese la colisin
                    newY = bloque.getCenY() + bloque.getAlto()/2+ pelota.getAlto()/2;
                    // Si no forzamos un movimiento brusco de la pelota  en el eje y 
                    if (Math.abs(pelota.getCenY() - newY) <= VelocidadAvanceY) {
                        // Fijamos la nueva posicin en el eje y
                    	pelota.setCenY(newY);
                    }
                    break;
                // Si la parte inferior de la pelota ha colisionado con el bloque
                case Bloque.COLISION_ABAJO:
                	// Si la pelota se est desplazando hacia abajo
                	if (velYPelota > 0) {
                		// Detenemos el movimiento vertical
                        velYPelota = 0;
                    }
                	// Obtenemos la nueva posicin de la pelota en el eje y,
                	// para que cese la colisin
                    newY = bloque.getCenY() - bloque.getAlto()/2 - pelota.getAlto()/2;
                    // Si no forzamos un movimiento brusco de la pelota en el eje y
                    if (Math.abs(pelota.getCenY() - newY) <= VelocidadAvanceY) {
                    	// Fijamos la nueva posicin en el eje y
                        pelota.setCenY(newY);
                    }
                    break;
            }

            switch (pelota.getColisionX()) {
            	// Si la parte izquierda de la pelota ha colisionado con el bloque
                case Bloque.COLISION_IZQUIERDA:
                	// Si la pelota se est desplazando hacia la izquierda
                    if (velXPelota < 0) {
                    	// Detenemos el movimiento horizontal
                        velXPelota = 0;
                    }
                    // Obtenemos la nueva posicin de la pelota en el eje x,
                	// para que cese la colisin
                    newX = bloque.getCenX() + bloque.getAncho() / 2 + pelota.getAncho() / 2;
                    // Si no forzamos un movimiento brusco de la pelota en el eje x
                    if (Math.abs(pelota.getCenX() - newX) <= VelocidadAvanceX) {
                    	// Fijamos la nueva posicin en el eje x
                    	pelota.setCenX(newX);
                    }
                    break;
                // Si la parte derecha de la pelota ha colisionado con el bloque
                case Bloque.COLISION_DERECHA:
                	// Si la pelota se est desplazando hacia la derecha
                	if (velXPelota > 0) {
                    	// Detenemos el movimiento horizontal
                        velXPelota = 0;
                    }
                	// Obtenemos la nueva posicin de la pelota en el eje x,
                	// para que cese la colisin
                    newX = bloque.getCenX() - bloque.getAncho()/2 - pelota.getAncho()/2;
                    // Si no forzamos un movimiento brusco de la pelota en el eje x 
                    if (Math.abs(pelota.getCenX() - newX) <= VelocidadAvanceX) {
                    	// Fijamos la nueva posicin en el eje x
                    	pelota.setCenX(newX);
                    }
                    break;
            }
        }
    }
    
    // Verificamos si hemos llegado a la meta
    private void verificaColisionPelotaMeta() {
    	// Si la pelota ha colisionado con la meta
		if (pelota.verificaColision(meta)) {
	    	// Activamos los sonidos, si hemos habilitado la msica por preferencias
			if (pref.getBoolean("musica", true)) {
				// Reproducimos el sonido asociado
				sounds.play(idSonidoMeta, 1, 1, 0, 0, 1);
			}
			
			// El terminal vibra
			Vibrator v = (Vibrator) this.context.getSystemService(Context.VIBRATOR_SERVICE); 
			v.vibrate(PERIODO_PROCESO);
			
			// Incrementamos la puntuacin
			puntuacion += 1000;
			
			// Si estamos en el ltimo nivel
			if (nivel == NUM_NIVELES) {
				// Salimos del juego
				salir();
			} else {
				// Incrementamos el nivel
				nivel++;
								
				// Establecemos el nuevo tipo de cada bloque
				estableceTiposBloques();
				
				// Eliminamos las monedas que queden
				eliminaMonedas();
				
				// Creamos las monedas de nuevo
				creaMonedas();

				// Posicionamos los elementos
				posicionaElementos();				
			}
		}
    }    
        
	public void activarSensores(){
		List<Sensor> listSensors;						

		// Si la pelota se maneja mediante el sensor de aceleracin,
		// lo activaremos si existe
		if (manejo == MANEJO_SENSOR_ACELER) {
			listSensors = mSensorManager.getSensorList(Sensor.TYPE_ACCELEROMETER);
			if (!listSensors.isEmpty()) {
				sensor = listSensors.get(0);
				mSensorManager.registerListener(this, sensor,
						SensorManager.SENSOR_DELAY_GAME);
			} else {
				// Mostramos un mensaje indicando que no se ha detectado el acelermetro
				Toast.makeText(context, context.getString(R.string.noAccelerometer),
						Toast.LENGTH_SHORT).show();
			}
		}													
	}

	public void desactivarSensores(){
		mSensorManager.unregisterListener(this);		
	}
	
	@Override 
	public void onAccuracyChanged(Sensor sensor, int accuracy) {}
	 	 
	@Override 
	public void onSensorChanged(SensorEvent event) {						
		switch(event.sensor.getType()) {		
			case Sensor.TYPE_ACCELEROMETER:
				float valorA_Y = event.values[1];  // Usamos sensor de eje Y
				float valorA_X = event.values[0];  // Usamos sensor de eje X

				// Si hemos movido el terminal segn el eje Y 
				if (valorA_Y > 0) {
					movimientoX = DERECHA;
				} else if (valorA_Y < 0) {
					movimientoX = IZQUIERDA;
				} else {
					movimientoX = QUIETO;
				}
				
				// Si hemos movido el terminal segn el eje X
 				if (valorA_X > 0) {
					movimientoY = ABAJO;
				} else if (valorA_X < 0) {
					movimientoY = ARRIBA;
				} else {
					movimientoY = QUIETO;
				}
 				
				break;				
		} 		
	}
	
	@Override
	public boolean onTouchEvent(MotionEvent event) {
		super.onTouchEvent(event);
				
		// Indicamos que, en principio,
		// no procesamos el evento
		boolean touchProcesado = false;		
		
		// Procesamos la pantalla tctil, si  
		// hemos habilitado la preferencia 
		if (manejo == MANEJO_TACTIL) {	
			// Indicamos que procesamos el evento 
			touchProcesado = true;
						
			switch (event.getAction()) {				
				case MotionEvent.ACTION_DOWN:				
				case MotionEvent.ACTION_MOVE:
					// Si hemos tocado la pantalla a la derecha de la pelota
					if (event.getX() > pelota.getCenX()) {
						// Indicamos que la pelota debe moverse hacia la derecha
						movimientoX = DERECHA;
					// Si hemos tocado la pantalla a la izquierda de la pelota
					} else if (event.getX() < pelota.getCenX()) {
						// Indicamos que la pelota debe moverse hacia la izquierda
						movimientoX = IZQUIERDA;
					} else {
						// Indicamos que la pelota no debe moverse en el eje x
						movimientoX = QUIETO;
					}
					
					// Si hemos tocado la pantalla por debajo de la pelota
					if (event.getY() > pelota.getCenY()) {
						// Indicamos que la pelota debe moverse hacia abajo
						movimientoY = ABAJO;
					// Si hemos tocado la pantalla por arriba de la pelota
					} else if (event.getY() < pelota.getCenY()) {
						// Indicamos que la pelota debe moverse hacia arriba
						movimientoY = ARRIBA;
					} else {
						// Indicamos que la pelota no debe moverse en el eje y
						movimientoY = QUIETO;
					}									
					break;										
				
				case MotionEvent.ACTION_UP:
					// Indicamos que la pelota debe detenerse
					movimientoX = QUIETO;
					movimientoY = QUIETO;
					break;								
			}			
		}
			
		return touchProcesado;
	}
       
	public void setPadre(Activity padre) {
        // Fijamos la actividad padre
		this.padre = padre;
    }

	public void salir() {
		// Si hemos activado la msica por preferencias
		if (pref.getBoolean("musica", true)) {
			// Liberamos los recursos
			sounds.release();
		}
		
		// Incluiremos la puntuacin en el bundle
        Bundle bundle = new Bundle();
		bundle.putInt("puntuacion", puntuacion);
		Intent intent = new Intent();
		intent.putExtras(bundle);
		padre.setResult(Activity.RESULT_OK, intent);
		padre.finish();
    }

    // Clase que implementa el thread
	class ThreadJuego extends Thread {
        private boolean pausa, corriendo;

        public synchronized void pausar() {
            pausa = true;
        }

        public synchronized void reanudar() {
            pausa = false;
            notify();
        }

        public void detener() {
            corriendo = false;
            if (pausa)
                reanudar();
        }

        @Override
        public void run() {
            corriendo = true;
            while (corriendo) {
                actualizaFisica();
                synchronized (this) {
                    while (pausa) {
                        try {
                            wait();
                        } catch (Exception e) {
                        }
                    }
                }
            }
        }
    }

    public ThreadJuego getThread() {
        return thread;
    }
}
